%% TP 3  -  SyS 2017

%// Autor: MMG

% Hay que conectar el micrfono y los parlantes antes de iniciar Matlab
% El objetivo de este script es analizar algunos sistemas lineales
% utilizando un vector producto de una grabacin de audio.

clear all; close all; clc;

%% 0) Grabando la seal de entrada x[n]:

format long
fs=20000;                  % frecuencia de muestreo en Hz
nb=16;                     % nmeros de bits usados por muestra
in=audiorecorder(fs,nb,1); % definimos el objeto de audio in
disp('Toque cualquier tecla para empezar a grabar')
pause
record(in)
disp('Toque cualquier tecla para terminar de grabar')
pause
stop(in)
disp('Toque cualquier tecla para reproducir')
pause
play(in)
x=getaudiodata(in,'double'); %convierte el objeto in a un vector con
                             %componentes reales entre -1 y 1

%% 1) Para empezar, pasaremos la seal de audio por un sistema que realiza 
% la reflexin de la seal de entrada. Es decir: y[n] = x[-n]. Luego se 
% reproduce el vector y[n] a la frecuencia de muestreo fs. Ya analizamos
% este sistema lineal en el ejercicio 2) f) del TP3.

inv = audioplayer(flipud(x),fs,nb); %flipud invierte las componentes del
%vector columna x (flip "up" and "down") y audioplayer convierte el vector
%invertido a un objeto con frecuencia de muestreo fs que se puede
%reproducir
play(inv)

%% 2) Veremos qu sucede al pasar la seal de audio por un SLID que 
% realiza el promedio mvil de N muestras de la seal de entrada

N=30;            % Nmero de muetras promediadas
h=ones(N,1)/N;   % Rta. impulsiona de este SLID. Ver Ejercicio 8 TP 3.
% fvtool(h)      % Herramienta para analizar la respuesta en frecuencia del 
                 % filtro. Vean las opciones que tiene y los distintos
                 % dibujos que les deja ver.
y=conv(h,x);     % Se calcula la salida del sistema.
y=y/max(abs(y)); % Se normaliza el vector para que quede entre -1 y 1.
out = audioplayer(y,fs,nb);
play(out)

% Qu efecto sonoro nota? Pruebe cambiar el nmero de muestras que se
% promedian y note los cambios. Tiene sentido lo que escucha? Qu 
% sucedera con frecuencias que sean mltiplos de fs/N? Quizs lo pueda ver
% en el grfico de la respuesta en magnitud utilizando 'fvtool(h)'.

%% 3) La idea ahora es pasar a la seal de audio por un sistema lineal que 
% provoque un filtrado 'pasa bajos' en la seal de audio x[n]. Esto tambin
% se conoce como 'filtrar' una seal. Para ello se especificar:
% - Orden del filtro (igual al nmero de trminos de la ecuacin en
% diferencias).
% - Frecuencia de corte del filtro.
% - Desviacin de la banda de paso mxima.
% - Desviacin en la banda de corte mxima.
% Para disear el filtro FIR solamente se deben especificar los 
% coeficientes de la ecuacin en diferencias. Para ello, se utiliza 
% 'fircls1' que utiliza  uno de los tantos mtodos existentes para 
% calcular los coeficientes ya mencionados (Cuadrados mnimos). 
% Teniendo la 'h' solo resta hacer la convolucin con la seal de audio y 
% escuchar el resultado.

clear y
n = 61;              % Orden del filtro pasa bajos
fc = 1000;           % Frecuencia de corte en Hz.
wc = 2*pi*fc;
wo = wc/(2*pi*fs/2); % Frecuencia de corte normalizada, wc*2/fs=wo
dp = 0.02;           % Mxima desviacin en la banda de paso (repecto a 1)
ds = 0.008;          % Mxima desviacin en la banda de corte (repecto a 0)

h = fircls1(n,wo,dp,ds);
fvtool(h,1)          % Para ver la respuesta en frecuencia del filtro.
pause
y=conv(h,x);
y=y/max(abs(y));
out = audioplayer(y,fs,nb);
play(out)

% - Cambie el orden del filtro y note las diferencias con la respuesta en
% frecuencia.
% - Grabe su voz y cambie la frecuencia de corte hasta que note una
% atenuacin significativa en la salida. Qu espectro aproximado posee 
% la voz? Es decir, cul es la frecuencia mxima (aproximadamente) para 
% su voz?


%% 4) Ahora, para analizar otro efecto, dupliquemos cada muestra de la 
% seal x[n]. Es decir, 

%           -
%   d[n] = |  x[n/2]  si n es par
%          |   0      si n es impar
%           -
%   y luego y[n] = d[n] + x[n-1]

% Por lo tanto, la salida de este sistema tendr un largo que ser el doble
% del de x[n] si el largo de x[n] es par y ser el doble + 1 si el largo 
% de x[n] es impar. 

for n=2:2:2*length(x);
    y(n-1)=x(n/2);
    y(n)=x(n/2);
end
out = audioplayer(y,fs,nb);
play(out)

% Como se puede escuchar, el resultado de reproducir y[n] a 'fs' es como
% reproducir x[n] a 'fs/2'.

%% 5) Como se adelant, cambiar la frecuencia a la que se reproduce el 
% audio provocar que el tiempo de muestreo cambie, lo que provocar que 
% el espectro del audio que escuchamos cambie. Si por ejemplo reducimos a 
% la mitad la frecuencia de reproduccin, escucharemos sonidos ms graves 
% que los que escuchamos antes. Por lo que hay una relacin recproca entre
% el dominio 'temporal' y el dominio de la 'frecuencia'.

out1 = audioplayer(x,fs/2,nb);
play(out1)
pause

% Si en vez de disminuir a la mitad, duplicamos la frecuencia de
% reproduccin, notaremos el efecto inverso.

out2 = audioplayer(x,2*fs,nb);
play(out2)

%% 6) Otro sistema (de particular inters en comunicaciones) es el en que 
% la salida resulta en la suma de entradas retardadas en t0, y cada uno de
% dichos trminos se encuentra multiplicado por una constante, que
% depender del ambiente donde se aplique el modelo. Por ejemplo, en
% comunicaciones se considera que la seal que arriva a la antena es la que
% debera ser, ms uno o ms trminos de dicha seal retardada un tiempo t0
% y atenuada en un factor A. Entonces una manera de modelar este sistema
% sera:
%           y[n] = x[n] + A1 * x[n-n0] + A2 * x[n-2*n0] + ....
%
% Se puede pensar que la respuesta impulsional de este sistema sern deltas
% de kronecker separadas n0, con amplitud igual a las constantes A.

clear y
t0 = 0.3;       % Retardo en tiempo para la seal de audio (segundos).
N=floor(fs*t0); % Para calcular cuntas muestras equivale dicho retardo t0.
h=[1;zeros(N-1,1);0.5;zeros(N-1,1);0.25;zeros(N-1,1);0.125]; % Rta. impulsional.
stem(h)
pause
y=conv(h,x);
y=y/max(abs(y));
out = audioplayer(y,fs,nb);
play(out)

% Cambie los valores de las A para que en vez de disminuir los rebotes,
% vayan aumentando la amplitud y escuche.

